package main

import (
	"fmt"
	"log"
	"net/http"
	"os/exec"
	"strconv"
	"strings"
	"time"

	"github.com/prometheus/client_golang/prometheus"
	"github.com/prometheus/client_golang/prometheus/promhttp"
)

// PORT is the TCP port number the HTTP server will listen to
var PORT = ":2345"

var (
	up1 = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "LXF",
			Name:      "uptime_1_minute",
			Help:      "Uptime 1 minute",
		})

	up5 = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "LXF",
			Name:      "uptime_5_minute",
			Help:      "Uptime 5 minutes",
		})

	up15 = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "LXF",
			Name:      "uptime_15_minute",
			Help:      "Uptime 15 minutes",
		})

	users = prometheus.NewGauge(
		prometheus.GaugeOpts{
			Namespace: "LXF",
			Name:      "number_of_users",
			Help:      "Number of users",
		})
)

func main() {
	http.Handle("/metrics", promhttp.Handler())
	prometheus.MustRegister(up1)
	prometheus.MustRegister(up5)
	prometheus.MustRegister(up15)
	prometheus.MustRegister(users)

	go func() {
		for {
			uptime, err := exec.Command("/usr/bin/uptime").Output()
			if err != nil {
				fmt.Println(err)
				return
			}

			// split uptime output
			t := strings.Split(string(uptime), "load average: ")[1]
			fields := strings.Split(t[:len(t)-1], ", ")

			// Extract the desired fields from uptime
			// Convert it to float64
			up1min, _ := strconv.ParseFloat(fields[:len(fields)][0], 64)
			up1.Set(up1min)

			// Convert it to float64
			up5min, _ := strconv.ParseFloat(fields[:len(fields)][1], 64)
			up5.Set(up5min)

			// Convert it to float64
			up15min, _ := strconv.ParseFloat(fields[:len(fields)][2], 64)
			up15.Set(up15min)

			// Get number of users
			u := strings.Split(string(uptime), " users,")
			if len(u) < 2 {
				temp := 0
				users.Set(float64(temp))
			} else {
				// Split again
				fields = strings.Split(u[0], " ")
				// Get the last field
				usr := fields[len(fields)-1]
				// Convert it to float64
				temp, _ := strconv.ParseFloat(usr, 64)
				users.Set(float64(temp))
			}

			time.Sleep(5 * time.Second)
		}
	}()

	log.Println("Listening to port", PORT)
	log.Println(http.ListenAndServe(PORT, nil))
}
